header {
package org.antlr.java.crossref;
import java.util.Stack;
}
/** Java 1.4 AST Recognizer Grammar
 *
 * Author: (see java.g preamble)
 *
 * This grammar is in the PUBLIC DOMAIN
 */
class JavaDefsTreeParser extends JavaTreeParser;

options {
	importVocab = Java;
    ASTLabelType = "org.antlr.java.crossref.LocationAST";
}

{
	static final boolean printDebug = false;
    FileScope fileScope = null;
//    DebugStack scopeStack = new DebugStack();
    Stack scopeStack = new Stack();
    CodeProject project = null;

    public void setProject(CodeProject project) {
        this.project = project;
    }

    protected String fileName = null;
    public void setFileName(String f) {
        fileName = f;
    }
}

compilationUnit
{
	String packageName=null;
}
    :
        (	packageName=packageDefinition
        {
	        project.mkPackageScopes(packageName);
	        PackageScope p = project.getPackageScope(packageName);
	        String fullName = new String(packageName + "." + fileName);
		    fileScope = new FileScope(project,fileName,null);
		    scopeStack.push(fileScope);
		    project.setFileScope(fullName, fileScope);
	        fileScope.setPackageScope(p);
	    //if(JavaDefsTreeParser.printDebug && CodeProject.printDebug)
        // System.out.println("JavaDefsTreeParser: " + "Define package: "+p.getFullyQualifiedName());
        }
        
		|	/* nothing */
		{
            project.mkPackageScopes("[default]");
	        PackageScope p = project.getPackageScope("[default]");
		    fileScope = new FileScope(project,fileName,p);
		    scopeStack.push(fileScope);
		    String fullName = "[default]." + fileName;
		    project.setFileScope(fullName, fileScope);
	        fileScope.setPackageScope(p);
		    //if(JavaDefsTreeParser.printDebug && CodeProject.printDebug)
		    // System.out.println("JavaDefsTreeParser: " + "Define FileScope: "+fileScope);
		}
		
		)
		(importDefinition)*
		(typeDefinition)*
		{scopeStack.pop();}
	;

packageDefinition  returns [String id = null]
	:	#( PACKAGE_DEF id=identifier )
	;

typeDefinition
	:	#(  CLASS_DEF modifiers ident:IDENT extendsClause implementsClause
            {
            Scope currentScope = (Scope)scopeStack.peek();
            ClassScope cs = null;
            //Change for predefined classes, i.e. the XML load
			cs = project.getClassScope(currentScope.getFullyQualifiedName() + "." + ident.getText());
			if(cs == null)
				cs = new ClassScope(project,ident.getText(),currentScope);
			else
				cs.setParent(currentScope);
            cs.setSourceLocation(fileName, ident.getLine(), ident.getColumn());
            currentScope.addScope(cs);
            scopeStack.push(cs);
            }
            objBlock
            {scopeStack.pop();}
         )
	|	#(  INTERFACE_DEF modifiers interf:IDENT extendsClause
            {
            ClassScope cs = null;
            Scope currentScope = (Scope)scopeStack.peek();
            //Change for predefined classes, i.e. the XML load
			cs = project.getClassScope(currentScope.getFullyQualifiedName() + "." + interf.getText());
			if(cs == null)
				cs = new ClassScope(project,interf.getText(),currentScope);
			else
				cs.setParent(currentScope);
            cs.setInterface(true);
            cs.setSourceLocation(fileName, interf.getLine(), interf.getColumn());
            currentScope.addScope(cs);
            scopeStack.push(cs);
            }
            interfaceBlock
            {scopeStack.pop();}
         )
	;

variableDeclarator
	:	varD:IDENT
        {
        Scope currentScope = (Scope)scopeStack.peek();
        if ( currentScope instanceof ClassScope ) {
            FieldVariable fs = new FieldVariable(project,varD.getText(),currentScope);
            fs.setSourceLocation(fileName, varD.getLine(), varD.getColumn());
            currentScope.addScopeMember(fs);
        }
        }
	|	LBRACK variableDeclarator
	;
/*
All this functionality moved to prerefs.g
overloaded methods and same-name methods with different signatures needed it.
See methodHead in prerefs.g, and refs.g for more explanation.
*/

ctorDef
	:	#(CTOR_DEF modifiers methodHead (slist)?)
        {scopeStack.pop();}
	;

methodDef
	:	#(METHOD_DEF modifiers typeSpec methodHead (slist)?)
        {scopeStack.pop();}
	;

methodDecl
	:	#(METHOD_DEF modifiers typeSpec methodHead)
        {scopeStack.pop();}
	;


methodHead
	:	methH:IDENT #( PARAMETERS (parameterDef)* ) (throwsClause)?
        {
            Scope currentScope = (Scope)scopeStack.peek();
            MethodScope ms = new MethodScope(project,methH.getText(),currentScope);
            ms.setSourceLocation(fileName, methH.getLine(), methH.getColumn());
//            currentScope.addScopeMember(ms);
//done in  currentScope.addScopeMember(ms);
//            project.setMethodScope(ms.getFullyQualifiedName(), ms);
			//Project is not informed about method scope
			//the push is done to prevent local fields from being connected to CLASS
			//if ( currentScope instanceof ClassScope ) {
			
            scopeStack.push(ms);
        }
	;

identifier returns [String id=null]
{
    String r = null;
}
	:	IDENT {id=#IDENT.getText();}
	|	#( DOT r=identifier IDENT ) {id=r+"."+#IDENT.getText();}
	;
